@@ -140,7 +140,7 @@ setting_vpn_add_data_item_utf8safe (NMSettingVpn *setting,
 	g_return_if_fail (value && value[0]);
 
 	nm_setting_vpn_add_data_item (setting, key,
-	                              nmv_utils_str_utf8safe_escape_c (value, &s));
+	                              nm_utils_str_utf8safe_escape (value, 0, &s));
 }
 
 static void
@@ -165,7 +165,7 @@ setting_vpn_eq_data_item_utf8safe (NMSettingVpn *setting,
 
 	if (!expected_value)
 		return FALSE;
-	return nm_streq (expected_value, nmv_utils_str_utf8safe_unescape_c (s, &s_free));
+	return nm_streq (expected_value, nm_utils_str_utf8safe_unescape (s, &s_free));
 }
 
 /*****************************************************************************/
@@ -794,7 +794,8 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 	gs_free char *basename = NULL;
 	gs_free char *default_path = NULL;
 	char *tmp, *tmp2;
-	const char *last_seen_key_direction = NULL;
+	const char *ta_direction = NULL, *secret_direction = NULL;
+	gboolean allow_ta_direction = FALSE, allow_secret_direction = FALSE;
 	gboolean have_certs, have_ca;
 	GSList *inline_blobs = NULL, *sl_iter;
 
@@ -871,8 +872,9 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_KEY_DIRECTION)) {
 			if (!args_params_check_nargs_n (params, 1, &line_error))
 				goto handle_line_error;
-			if (!args_params_parse_key_direction (params, 1, &last_seen_key_direction, &line_error))
+			if (!args_params_parse_key_direction (params, 1, &ta_direction, &line_error))
 				goto handle_line_error;
+			secret_direction = ta_direction;
 			continue;
 		}
 
@@ -1069,6 +1071,8 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			const char *prev;
 			GString *new_remote;
 			int port = -1;
+			gboolean host_has_colon;
+			struct in6_addr a;
 
 			if (!args_params_check_nargs_minmax (params, 1, 3, &line_error))
 				goto handle_line_error;
@@ -1090,8 +1094,8 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				port = v_int64;
 
 				if (params[3]) {
-					if (!NM_IN_STRSET (params[3], "udp", "tcp")) {
-						line_error = g_strdup_printf (_("remote expects protocol either udp or remote"));
+					if (!NM_IN_STRSET (params[3], NMOVPN_PROTCOL_TYPES)) {
+						line_error = g_strdup_printf (_("remote expects protocol type like “udp” or “tcp”"));
 						goto handle_line_error;
 					}
 				}
@@ -1104,14 +1108,25 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				g_string_assign (new_remote, prev);
 				g_string_append (new_remote, ", ");
 			}
-			g_string_append (new_remote, params[1]);
+
+			host_has_colon = (strchr (params[1], ':') != NULL);
+			if (   host_has_colon
+			    && inet_pton (AF_INET6, params[1], &a) == 1) {
+				/* need to escape the host. */
+				g_string_append_printf (new_remote, "[%s]", params[1]);
+			} else
+				g_string_append (new_remote, params[1]);
+
 			if (params[2]) {
 				g_string_append_printf (new_remote, ":%d", port);
 				if (params[3]) {
 					g_string_append_c (new_remote, ':');
 					g_string_append (new_remote, params[3]);
-				}
-			}
+				} else if (host_has_colon)
+					g_string_append_c (new_remote, ':');
+			} else if (host_has_colon)
+				g_string_append (new_remote, "::");
+
 			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE, new_remote->str);
 			g_string_free (new_remote, TRUE);
 
@@ -1175,8 +1190,8 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			const char *s_direction = NULL;
 
 			can_have_direction = NM_IN_STRSET (params[0],
-			                                  NMV_OVPN_TAG_SECRET,
-			                                  NMV_OVPN_TAG_TLS_AUTH);
+			                                   NMV_OVPN_TAG_SECRET,
+			                                   NMV_OVPN_TAG_TLS_AUTH);
 
 			if (!args_params_check_nargs_minmax (params, 1, can_have_direction ? 2 : 1, &line_error))
 				goto handle_line_error;
@@ -1188,7 +1203,6 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			if (params[2]) {
 				if (!args_params_parse_key_direction (params, 2, &s_direction, &line_error))
 					goto handle_line_error;
-				last_seen_key_direction = s_direction;
 			}
 
 			if (!g_path_is_absolute (file))
@@ -1207,12 +1221,14 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_SECRET)) {
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_STATIC_KEY, file);
 				if (s_direction)
-					setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, s_direction);
+					secret_direction = s_direction;
+				allow_secret_direction = TRUE;
 				have_sk = TRUE;
 			} else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_AUTH)) {
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_TA, file);
 				if (s_direction)
-					setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_TA_DIR, s_direction);
+					ta_direction = s_direction;
+				allow_ta_direction = TRUE;
 			} else if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_CRYPT))
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_TLS_CRYPT, file);
 			else
@@ -1405,7 +1421,6 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			gboolean is_base64 = FALSE;
 			char *f_path;
 			const char *key;
-			gboolean can_have_direction = FALSE;
 			GString *blob_data;
 			InlineBlobData *inline_blob_data;
 
@@ -1422,10 +1437,10 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				key = NM_OPENVPN_KEY_TLS_CRYPT;
 			else if (nm_streq (token, INLINE_BLOB_TLS_AUTH)) {
 				key = NM_OPENVPN_KEY_TA;
-				can_have_direction = TRUE;
+				allow_ta_direction = TRUE;
 			} else if (nm_streq (token, INLINE_BLOB_SECRET)) {
 				key = NM_OPENVPN_KEY_STATIC_KEY;
-				can_have_direction = TRUE;
+				allow_secret_direction = TRUE;
 			} else {
 				line_error = g_strdup_printf (_("unsupported blob/xml element"));
 				goto handle_line_error;
@@ -1502,9 +1517,6 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_CERT, f_path);
 				setting_vpn_add_data_item_path (s_vpn, NM_OPENVPN_KEY_KEY, f_path);
 			}
-			if (   can_have_direction
-			    && last_seen_key_direction)
-				setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_TA_DIR, last_seen_key_direction);
 			continue;
 		}
 
@@ -1522,6 +1534,11 @@ handle_line_error:
 		goto out_error;
 	}
 
+	if (allow_secret_direction && secret_direction)
+		setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, secret_direction);
+	if (allow_ta_direction && ta_direction)
+		setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_TA_DIR, ta_direction);
+
 	if (!have_client && !have_sk) {
 		g_set_error_literal (error,
 		                     NMV_EDITOR_PLUGIN_ERROR,
@@ -1577,7 +1594,7 @@ handle_line_error:
 		const char *key_path;
 
 		key_path = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
-		if (is_encrypted (nmv_utils_str_utf8safe_unescape_c (key_path, &key_path_free))) {
+		if (is_encrypted (nm_utils_str_utf8safe_unescape (key_path, &key_path_free))) {
 			/* If there should be a private key password, default it to
 			 * being agent-owned.
 			 */
@@ -1796,32 +1813,25 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 	args_write_line (f, NMV_OVPN_TAG_CLIENT);
 
 	/* 'remote' */
-	gw_list = g_strsplit_set (gateways, " ,", 0);
+	gw_list = g_strsplit_set (gateways, " \t,", 0);
 	for (gw_iter = gw_list; gw_iter && *gw_iter; gw_iter++) {
-		char *tmp_host, *tmp_port,*tmp_proto;
+		gs_free char *str_free = NULL;
+		const char *host, *port, *proto;
+		gssize eidx;
 
-		if (**gw_iter == '\0')
+		eidx = nmovpn_remote_parse (*gw_iter,
+		                            &str_free,
+		                            &host,
+		                            &port,
+		                            &proto,
+		                            NULL);
+		if (eidx >= 0)
 			continue;
-		tmp_host = g_strstrip (*gw_iter);
-		tmp_port = strchr (tmp_host, ':');
-		tmp_proto = tmp_port ? strchr (tmp_port + 1, ':') : NULL;
-		if (tmp_port)
-			*tmp_port++ = '\0';
-		if (tmp_proto)
-			*tmp_proto++ = '\0';
-		if (tmp_port && !*tmp_port)
-			tmp_port = NULL;
-		if (tmp_proto && !*tmp_proto)
-			tmp_proto = NULL;
-
 		args_write_line (f,
 		                 NMV_OVPN_TAG_REMOTE,
-		                 *gw_iter,
-		                 tmp_port
-		                     ?: (tmp_proto
-		                             ? (nm_streq (tmp_proto, "udp") ? "1194" : "443")
-		                             : NULL),
-		                 tmp_proto);
+		                 host,
+		                 port ?: (proto ? "1194" : NULL),
+		                 proto);
 	}
 	g_strfreev (gw_list);
 
@@ -1840,18 +1850,18 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 		                                   NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
 			value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA);
 			if (_arg_is_set (value))
-				cacert = nmv_utils_str_utf8safe_unescape_c (value, &cacert_free);
+				cacert = nm_utils_str_utf8safe_unescape (value, &cacert_free);
 		}
 
 		if (NM_IN_STRSET (connection_type, NM_OPENVPN_CONTYPE_TLS,
 		                                   NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
 			value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
 			if (_arg_is_set (value))
-				user_cert = nmv_utils_str_utf8safe_unescape_c (value, &user_cert_free);
+				user_cert = nm_utils_str_utf8safe_unescape (value, &user_cert_free);
 
 			value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
 			if (_arg_is_set (value))
-				private_key = nmv_utils_str_utf8safe_unescape_c (value, &private_key_free);
+				private_key = nm_utils_str_utf8safe_unescape (value, &private_key_free);
 		}
 
 		if (   cacert && user_cert && private_key
@@ -1879,7 +1889,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 			args_write_line (f,
 			                 NMV_OVPN_TAG_SECRET,
-			                 nmv_utils_str_utf8safe_unescape_c (value, &s_free),
+			                 nm_utils_str_utf8safe_unescape (value, &s_free),
 			                 _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION)));
 		}
 	}
@@ -1922,7 +1932,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 		device_type = _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_DEV_TYPE));
 		device = _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_DEV));
-		device = nmv_utils_str_utf8safe_unescape_c (device, &device_free);
+		device = nm_utils_str_utf8safe_unescape (device, &device_free);
 		args_write_line (f,
 		                 NMV_OVPN_TAG_DEV,
 		                 device ?:
@@ -1980,7 +1990,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 			gs_free char *s_free = NULL;
 			args_write_line (f,
 			                 NMV_OVPN_TAG_TLS_AUTH,
-			                 nmv_utils_str_utf8safe_unescape_c (key, &s_free),
+			                 nm_utils_str_utf8safe_unescape (key, &s_free),
 			                 _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TA_DIR)));
 		}
 
@@ -1989,7 +1999,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 			gs_free char *s_free = NULL;
 			args_write_line (f,
 			                 NMV_OVPN_TAG_TLS_CRYPT,
-			                 nmv_utils_str_utf8safe_unescape_c (key, &s_free));
+			                 nm_utils_str_utf8safe_unescape (key, &s_free));
 		}
 
 	}
