@@ -37,6 +37,7 @@
 #include <gtk/gtk.h>
 
 #include "auth-helpers.h"
+#include "utils.h"
 
 /*****************************************************************************/
 
@@ -51,11 +52,11 @@ G_DEFINE_TYPE_EXTENDED (OpenvpnEditor, openvpn_editor_plugin_widget, G_TYPE_OBJE
 typedef struct {
 	GtkBuilder *builder;
 	GtkWidget *widget;
-	GtkSizeGroup *group;
 	GtkWindowGroup *window_group;
 	gboolean window_added;
 	GHashTable *advanced;
 	gboolean new_connection;
+	GtkWidget *tls_user_cert_chooser;
 } OpenvpnEditorPrivate;
 
 /*****************************************************************************/
@@ -64,48 +65,30 @@ typedef struct {
 #define COL_AUTH_PAGE 1
 #define COL_AUTH_TYPE 2
 
-/* Example: abc.com:1234:udp, ovpnserver.company.com:443, vpn.example.com::tcp */
 static gboolean
 check_gateway_entry (const char *str)
 {
-	char **list, **iter;
-	char *host, *port, *proto;
-	long int port_int;
+	gs_free char *str_clone = NULL;
+	char *str_iter;
+	const char *tok;
 	gboolean success = FALSE;
 
-	if (!str || !*str)
+	if (!str || !str[0])
 		return FALSE;
 
-	list = g_strsplit_set (str, " \t,", -1);
-	for (iter = list; iter && *iter; iter++) {
-		if (!**iter)
-			continue;
-		host = g_strstrip (*iter);
-		port = strchr (host, ':');
-		proto = port ? strchr (port + 1, ':') : NULL;
-		if (port)
-			*port++ = '\0';
-		if (proto)
-			*proto++ = '\0';
-
-		/* check hostname */
-		if (!*host)
-			goto out;
-		/* check port */
-		if (port && *port) {
-			char *end;
-			errno = 0;
-			port_int = strtol (port, &end, 10);
-			if (errno != 0 || *end != '\0' || port_int < 1 || port_int > 65535)
-				goto out;
-		}
-		/* check proto */
-		if (proto && strcmp (proto, "udp") && strcmp (proto, "tcp"))
-			goto out;
+	str_clone = g_strdup (str);
+	str_iter = str_clone;
+	while ((tok = strsep (&str_iter, " \t,"))) {
+		if (   tok[0]
+		    && (nmovpn_remote_parse (tok,
+		                             NULL,
+		                             NULL,
+		                             NULL,
+		                             NULL,
+		                             NULL) != -1))
+		   return FALSE;
+		success = TRUE;
 	}
-	success = TRUE;
-out:
-	g_strfreev (list);
 	return success;
 }
 
@@ -118,18 +101,14 @@ check_validity (OpenvpnEditor *self, GError **error)
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	const char *contype = NULL;
-	GdkRGBA rgba;
-	gboolean gateway_valid;
 	gboolean success;
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "gateway_entry"));
 	str = gtk_entry_get_text (GTK_ENTRY (widget));
-	gateway_valid = check_gateway_entry (str);
-	/* Change entry background colour while editing */
-	if (!gateway_valid)
-		gdk_rgba_parse (&rgba, "red3");
-	gtk_widget_override_background_color (widget, GTK_STATE_FLAG_NORMAL, !gateway_valid ? &rgba : NULL);
-	if (!gateway_valid) {
+	if (str && check_gateway_entry (str))
+		gtk_style_context_remove_class (gtk_widget_get_style_context (widget), "error");
+	else {
+		gtk_style_context_add_class (gtk_widget_get_style_context (widget), "error");
 		g_set_error (error,
 		             NMV_EDITOR_PLUGIN_ERROR,
 		             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
@@ -161,15 +140,12 @@ auth_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 	OpenvpnEditor *self = OPENVPN_EDITOR (user_data);
 	OpenvpnEditorPrivate *priv = OPENVPN_EDITOR_GET_PRIVATE (self);
 	GtkWidget *auth_notebook;
-	GtkWidget *show_passwords;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	gint new_page = 0;
 
 	auth_notebook = GTK_WIDGET (gtk_builder_get_object (priv->builder, "auth_notebook"));
 	g_assert (auth_notebook);
-	show_passwords = GTK_WIDGET (gtk_builder_get_object (priv->builder, "show_passwords"));
-	g_assert (auth_notebook);
 
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
 	g_assert (model);
@@ -177,9 +153,6 @@ auth_combo_changed_cb (GtkWidget *combo, gpointer user_data)
 
 	gtk_tree_model_get (model, &iter, COL_AUTH_PAGE, &new_page, -1);
 
-	/* Static key page doesn't have any passwords */
-	gtk_widget_set_sensitive (show_passwords, new_page != 3);
-
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (auth_notebook), new_page);
 
 	stuff_changed_cb (combo, self);
@@ -270,11 +243,8 @@ init_editor_plugin (OpenvpnEditor *self, NMConnection *connection, GError **erro
 
 	s_vpn = nm_connection_get_setting_vpn (connection);
 
-	priv->group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "gateway_entry"));
 	g_return_val_if_fail (widget != NULL, FALSE);
-	gtk_size_group_add_widget (priv->group, widget);
 	if (s_vpn) {
 		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE);
 		if (value)
@@ -284,7 +254,6 @@ init_editor_plugin (OpenvpnEditor *self, NMConnection *connection, GError **erro
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "auth_combo"));
 	g_return_val_if_fail (widget != NULL, FALSE);
-	gtk_size_group_add_widget (priv->group, widget);
 
 	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING);
 
@@ -301,7 +270,7 @@ init_editor_plugin (OpenvpnEditor *self, NMConnection *connection, GError **erro
 	}
 
 	/* TLS auth widget */
-	tls_pw_init_auth_widget (priv->builder, priv->group, s_vpn,
+		tls_pw_init_auth_widget (priv->builder, s_vpn,
 	                         NM_OPENVPN_CONTYPE_TLS, "tls",
 	                         stuff_changed_cb, self);
 	gtk_list_store_append (store, &iter);
@@ -312,7 +281,7 @@ init_editor_plugin (OpenvpnEditor *self, NMConnection *connection, GError **erro
 	                    -1);
 
 	/* Password auth widget */
-	tls_pw_init_auth_widget (priv->builder, priv->group, s_vpn,
+	tls_pw_init_auth_widget (priv->builder, s_vpn,
 	                         NM_OPENVPN_CONTYPE_PASSWORD, "pw",
 	                         stuff_changed_cb, self);
 	gtk_list_store_append (store, &iter);
@@ -325,7 +294,7 @@ init_editor_plugin (OpenvpnEditor *self, NMConnection *connection, GError **erro
 		active = 1;
 
 	/* Password+TLS auth widget */
-	tls_pw_init_auth_widget (priv->builder, priv->group, s_vpn,
+	tls_pw_init_auth_widget (priv->builder, s_vpn,
 	                         NM_OPENVPN_CONTYPE_PASSWORD_TLS, "pw_tls",
 	                         stuff_changed_cb, self);
 	gtk_list_store_append (store, &iter);
@@ -338,7 +307,7 @@ init_editor_plugin (OpenvpnEditor *self, NMConnection *connection, GError **erro
 		active = 2;
 
 	/* Static key auth widget */
-	sk_init_auth_widget (priv->builder, priv->group, s_vpn, stuff_changed_cb, self);
+	sk_init_auth_widget (priv->builder, s_vpn, stuff_changed_cb, self);
 
 	gtk_list_store_append (store, &iter);
 	gtk_list_store_set (store, &iter,
@@ -412,7 +381,8 @@ update_connection (NMVpnEditor *iface,
 	OpenvpnEditorPrivate *priv = OPENVPN_EDITOR_GET_PRIVATE (self);
 	NMSettingVpn *s_vpn;
 	GtkWidget *widget;
-	char *str, *auth_type;
+	char *auth_type;
+	const char *str;
 	gboolean valid = FALSE;
 
 	if (!check_validity (self, error))
@@ -421,10 +391,9 @@ update_connection (NMVpnEditor *iface,
 	s_vpn = NM_SETTING_VPN (nm_setting_vpn_new ());
 	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_VPN_SERVICE_TYPE_OPENVPN, NULL);
 
-	/* Gateway */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "gateway_entry"));
-	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
-	if (str && strlen (str))
+	str = gtk_entry_get_text (GTK_ENTRY (widget));
+	if (str && str[0])
 		nm_setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE, str);
 
 	auth_type = get_auth_type (priv->builder);
@@ -488,7 +457,6 @@ openvpn_editor_new (NMConnection *connection, GError **error)
 {
 	NMVpnEditor *object;
 	OpenvpnEditorPrivate *priv;
-	char *ui_file;
 	gboolean new = TRUE;
 	NMSettingVpn *s_vpn;
 
@@ -503,29 +471,20 @@ openvpn_editor_new (NMConnection *connection, GError **error)
 
 	priv = OPENVPN_EDITOR_GET_PRIVATE (object);
 
-	ui_file = g_strdup_printf ("%s/%s", UIDIR, "nm-openvpn-dialog.ui");
 	priv->builder = gtk_builder_new ();
 
 	gtk_builder_set_translation_domain (priv->builder, GETTEXT_PACKAGE);
 
-	if (!gtk_builder_add_from_file (priv->builder, ui_file, error)) {
-		g_warning ("Couldn't load builder file: %s",
-		           error && *error ? (*error)->message : "(unknown)");
-		g_clear_error (error);
-		g_set_error (error, NMV_EDITOR_PLUGIN_ERROR, 0,
-		             "could not load required resources from %s", ui_file);
-		g_free (ui_file);
+	if (!gtk_builder_add_from_resource (priv->builder, "/org/freedesktop/network-manager-openvpn/nm-openvpn-dialog.ui", error)) {
 		g_object_unref (object);
-		return NULL;
+		g_return_val_if_reached (NULL);
 	}
 
-	g_free (ui_file);
-
 	priv->widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "openvpn-vbox"));
 	if (!priv->widget) {
 		g_set_error_literal (error, NMV_EDITOR_PLUGIN_ERROR, 0, _("could not load UI widget"));
 		g_object_unref (object);
-		return NULL;
+		g_return_val_if_reached (NULL);
 	}
 	g_object_ref_sink (priv->widget);
 
@@ -556,8 +515,6 @@ dispose (GObject *object)
 	OpenvpnEditor *plugin = OPENVPN_EDITOR (object);
 	OpenvpnEditorPrivate *priv = OPENVPN_EDITOR_GET_PRIVATE (plugin);
 
-	g_clear_object (&priv->group);
-
 	g_clear_object (&priv->window_group);
 
 	g_clear_object (&priv->widget);
