@@ -43,6 +43,71 @@
 
 /*****************************************************************************/
 
+static void
+_test_nmovpn_remote_parse (const char *str,
+                           const char *exp_host,
+                           const char *exp_port,
+                           const char *exp_proto)
+{
+	gs_free char *str_free = NULL;
+	gssize r;
+	const char *host, *port, *proto;
+	gs_free_error GError *error = NULL;
+
+	g_assert (exp_host || (!exp_port && !exp_proto));
+
+	r = nmovpn_remote_parse (str, &str_free, &host, &port, &proto, &error);
+	if (!exp_host) {
+		g_assert (r >= 0);
+		g_assert (error);
+		return;
+	}
+	nmtst_assert_success (r == -1, error);
+
+	g_assert_cmpstr (exp_host, ==, host);
+	g_assert_cmpstr (exp_port, ==, port);
+	g_assert_cmpstr (exp_proto, ==, proto);
+}
+
+static void
+test_nmovpn_remote_parse (void)
+{
+	_test_nmovpn_remote_parse ("a",                          "a",                      NULL,    NULL);
+	_test_nmovpn_remote_parse ("a:",                         "a",                      NULL,    NULL);
+	_test_nmovpn_remote_parse ("t::",                        "t",                      NULL,    NULL);
+	_test_nmovpn_remote_parse ("a::",                        "a::",                    NULL,    NULL);
+	_test_nmovpn_remote_parse ("[a::]:",                     "a::",                    NULL,    NULL);
+	_test_nmovpn_remote_parse ("t:::",                       "t:",                     NULL,    NULL);
+	_test_nmovpn_remote_parse ("a:::",                       "a::",                    NULL,    NULL);
+	_test_nmovpn_remote_parse ("a:t::",                      "a:t",                    NULL,    NULL);
+	_test_nmovpn_remote_parse ("a:b::",                      "a:b::",                  NULL,    NULL);
+	_test_nmovpn_remote_parse ("a::udp",                     "a",                      NULL,    "udp");
+	_test_nmovpn_remote_parse ("a:1:",                       "a",                      "1",     NULL);
+	_test_nmovpn_remote_parse ("t::1:",                      "t:",                     "1",     NULL);
+	_test_nmovpn_remote_parse ("t::1:",                      "t:",                     "1",     NULL);
+	_test_nmovpn_remote_parse ("[a:]:1:",                    "[a:]",                   "1",     NULL);
+	_test_nmovpn_remote_parse ("a::1:",                      "a::1",                   NULL,    NULL);
+	_test_nmovpn_remote_parse ("a::1:1194",                  "a::1:1194",              NULL,    NULL);
+	_test_nmovpn_remote_parse ("[a::1]:1194",                "a::1",                   "1194",  NULL);
+	_test_nmovpn_remote_parse ("a::1194",                    "a::1194",                NULL,    NULL);
+	_test_nmovpn_remote_parse ("a::1194:",                   "a::1194",                NULL,    NULL);
+	_test_nmovpn_remote_parse ("[a:]:1194:",                 "[a:]",                   "1194",  NULL);
+	_test_nmovpn_remote_parse ("a:1:tcp",                    "a",                      "1",     "tcp");
+	_test_nmovpn_remote_parse ("aa:bb::1:1194:udp",          NULL,                     NULL,    NULL);
+	_test_nmovpn_remote_parse ("[aa:bb::1]:1194:udp",        "aa:bb::1",               "1194",  "udp");
+	_test_nmovpn_remote_parse ("[aa:bb::1]::udp",            "aa:bb::1",               NULL,    "udp");
+	_test_nmovpn_remote_parse ("aa:bb::1::udp",              "aa:bb::1",               NULL,    "udp");
+	_test_nmovpn_remote_parse ("aa:bb::1::",                 "aa:bb::1",               NULL,    NULL);
+	_test_nmovpn_remote_parse ("abc.com:1234:udp",           "abc.com",                "1234",  "udp");
+	_test_nmovpn_remote_parse ("ovpnserver.company.com:443", "ovpnserver.company.com", "443",   NULL);
+	_test_nmovpn_remote_parse ("vpn.example.com::tcp",       "vpn.example.com",        NULL,    "tcp");
+	_test_nmovpn_remote_parse ("dead:beef::1:1194",          "dead:beef::1:1194",      NULL,    NULL);
+	_test_nmovpn_remote_parse ("dead:beef::1:1194",          "dead:beef::1:1194",      NULL,    NULL);
+	_test_nmovpn_remote_parse ("2001:dead:beef::1194::",     "2001:dead:beef::1194",   NULL,    NULL);
+}
+
+/*****************************************************************************/
+
 static NMVpnEditorPlugin *
 _create_plugin (void)
 {
@@ -523,16 +588,18 @@ test_non_utf8_import (void)
 }
 
 static void
-test_static_key_import (void)
+test_static_key_import (gconstpointer test_data)
 {
 	_CREATE_PLUGIN (plugin);
 	NMConnection *connection;
 	NMSettingConnection *s_con;
 	NMSettingVpn *s_vpn;
-	const char *expected_id = "static";
+	const char *file, *expected_id, *expected_dir;
 	char *expected_path;
 
-	connection = get_basic_connection (plugin, SRCDIR, "static.ovpn");
+	nmtst_test_data_unpack (test_data, &file, &expected_id, &expected_dir);
+
+	connection = get_basic_connection (plugin, SRCDIR, file);
 	g_assert (connection);
 
 	/* Connection setting */
@@ -554,7 +621,7 @@ test_static_key_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "10.11.12.13");
 	_check_item (s_vpn, NM_OPENVPN_KEY_PORT, NULL);
-	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, "1");
+	_check_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, expected_dir);
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_TA_DIR, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_CIPHER, NULL);
@@ -676,7 +743,7 @@ test_proxy_http_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_FLOAT, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, "0");
-	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "test.server.com:443");
+	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "[aa:bb::1]:1194:udp");
 	_check_item (s_vpn, NM_OPENVPN_KEY_PORT, "2352");
 	_check_item (s_vpn, NM_OPENVPN_KEY_CERT, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_KEY, NULL);
@@ -1075,81 +1142,6 @@ test_args_parse_line (void)
 
 /*****************************************************************************/
 
-static void
-do_test_utils_str_utf8safe (const char *str, const char *expected)
-{
-	const char *str_safe, *s;
-	gs_free char *str2 = NULL;
-	gs_free char *str3 = NULL;
-
-	str_safe = nmv_utils_str_utf8safe_escape_c (str, &str2);
-
-	str3 = nmv_utils_str_utf8safe_escape (str);
-	g_assert_cmpstr (str3, ==, str_safe);
-	g_assert ((!str && !str3) || (str != str3));
-	g_clear_pointer (&str3, g_free);
-
-	if (expected == NULL) {
-		g_assert (str_safe == str);
-		g_assert (!str2);
-		if (str) {
-			g_assert (!strchr (str, '\\'));
-			g_assert (g_utf8_validate (str, -1, NULL));
-		}
-
-		g_assert (str == nmv_utils_str_utf8safe_unescape_c (str_safe, &str3));
-		g_assert (!str3);
-
-		str3 = nmv_utils_str_utf8safe_unescape (str_safe);
-		if (str) {
-			g_assert (str3 != str);
-			g_assert_cmpstr (str3, ==, str);
-		} else
-			g_assert (!str3);
-		g_clear_pointer (&str3, g_free);
-		return;
-	}
-
-	g_assert (str);
-	g_assert (str_safe != str);
-	g_assert (str_safe == str2);
-	g_assert (strchr (str, '\\') || !g_utf8_validate (str, -1, NULL));
-	g_assert (g_utf8_validate (str_safe, -1, NULL));
-
-	str3 = g_strcompress (str_safe);
-	g_assert_cmpstr (str, ==, str3);
-	g_clear_pointer (&str3, g_free);
-
-	str3 = nmv_utils_str_utf8safe_unescape (str_safe);
-	g_assert (str3 != str);
-	g_assert_cmpstr (str3, ==, str);
-	g_clear_pointer (&str3, g_free);
-
-	s = nmv_utils_str_utf8safe_unescape_c (str_safe, &str3);
-	g_assert (str3 != str);
-	g_assert (s == str3);
-	g_assert_cmpstr (str3, ==, str);
-	g_clear_pointer (&str3, g_free);
-
-	g_assert_cmpstr (str_safe, ==, expected);
-}
-
-static void
-test_utils_str_utf8safe (void)
-{
-	do_test_utils_str_utf8safe (NULL, NULL);
-	do_test_utils_str_utf8safe ("", NULL);
-	do_test_utils_str_utf8safe ("a", NULL);
-	do_test_utils_str_utf8safe ("ab", NULL);
-	do_test_utils_str_utf8safe ("abäb", NULL);
-	do_test_utils_str_utf8safe ("㈞abä㈞b", NULL);
-	do_test_utils_str_utf8safe ("Äab\\äb", "Äab\\\\äb");
-	do_test_utils_str_utf8safe ("ÄÄab\\äb", "ÄÄab\\\\äb");
-	do_test_utils_str_utf8safe ("Ä\304ab\\äb", "Ä\\304ab\\\\äb");
-}
-
-/*****************************************************************************/
-
 NMTST_DEFINE ();
 
 int main (int argc, char **argv)
@@ -1169,6 +1161,8 @@ int main (int argc, char **argv)
 #define _add_test_func_simple(func)       g_test_add_func ("/ovpn/properties/" #func, func)
 #define _add_test_func(detail, func, ...) nmtst_add_test_func ("/ovpn/properties/" detail, func, ##__VA_ARGS__)
 
+	_add_test_func_simple (test_nmovpn_remote_parse);
+
 	_add_test_func_simple (test_password_import);
 	_add_test_func ("password-export", test_export_compare, "password.conf", "password.ovpntest");
 
@@ -1184,7 +1178,8 @@ int main (int argc, char **argv)
 
 	_add_test_func_simple (test_non_utf8_import);
 
-	_add_test_func_simple (test_static_key_import);
+	_add_test_func ("static-import-1", test_static_key_import, "static.ovpn", "static", "1");
+	_add_test_func ("static-import-2", test_static_key_import, "static2.ovpn", "static2", "0");
 	_add_test_func ("static", test_export_compare, "static.ovpn", "static.ovpntest");
 
 	_add_test_func ("port-import", test_port_import, "port.ovpn", "port", "2345");
@@ -1230,8 +1225,6 @@ int main (int argc, char **argv)
 
 	_add_test_func_simple (test_args_parse_line);
 
-	_add_test_func_simple (test_utils_str_utf8safe);
-
 	result = g_test_run ();
 	if (result != EXIT_SUCCESS)
 		return result;
