@@ -1061,7 +1061,7 @@ add_openvpn_arg_utf8safe (GPtrArray *args, const char *arg)
 	g_return_val_if_fail (args, NULL);
 	g_return_val_if_fail (arg, NULL);
 
-	arg_unescaped = nmv_utils_str_utf8safe_unescape (arg);
+	arg_unescaped = nm_utils_str_utf8safe_unescape_cp (arg);
 	g_ptr_array_add (args, arg_unescaped);
 	return arg_unescaped;
 }
@@ -1098,9 +1098,9 @@ add_cert_args (GPtrArray *args, NMSettingVpn *s_vpn)
 	cert = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
 	key  = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
 
-	ca   = nmv_utils_str_utf8safe_unescape_c (ca,   &ca_free);
-	cert = nmv_utils_str_utf8safe_unescape_c (cert, &cert_free);
-	key  = nmv_utils_str_utf8safe_unescape_c (key,  &key_free);
+	ca   = nm_utils_str_utf8safe_unescape (ca,   &ca_free);
+	cert = nm_utils_str_utf8safe_unescape (cert, &cert_free);
+	key  = nm_utils_str_utf8safe_unescape (key,  &key_free);
 
 	if (   ca && strlen (ca)
 	    && cert && strlen (cert)
@@ -1330,22 +1330,25 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	if (tmp && *tmp) {
 		gs_free char *tmp_clone = NULL;
 		char *tmp_remaining;
-		char *tok, *port, *proto;
+		const char *tok;
 
 		tmp_remaining = tmp_clone = g_strdup (tmp);
 		while ((tok = strsep (&tmp_remaining, " \t,")) != NULL) {
-			if (!*tok)
+			gs_free char *str_free = NULL;
+			const char *host, *port, *proto;
+			gssize eidx;
+
+			eidx = nmovpn_remote_parse (tok,
+			                            &str_free,
+			                            &host,
+			                            &port,
+			                            &proto,
+			                            NULL);
+			if (eidx >= 0)
 				continue;
 
-			port = strchr (tok, ':');
-			proto = port ? strchr (port + 1, ':') : NULL;
-			if (port)
-				*port++ = '\0';
-			if (proto)
-				*proto++ = '\0';
-
 			add_openvpn_arg (args, "--remote");
-			add_openvpn_arg (args, tok);
+			add_openvpn_arg (args, host);
 			if (port) {
 				if (!add_openvpn_arg_int (args, port)) {
 					g_set_error (error,
@@ -1367,10 +1370,14 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 				add_openvpn_arg (args, "1194"); /* default IANA port */
 
 			if (proto) {
-				if (!strcmp (proto, "udp"))
-					add_openvpn_arg (args, proto);
-				else if (!strcmp (proto, "tcp"))
+				if (nm_streq (proto, "tcp"))
 					add_openvpn_arg (args, "tcp-client");
+				else if (nm_streq (proto, "tcp4"))
+					add_openvpn_arg (args, "tcp4-client");
+				else if (nm_streq (proto, "tcp6"))
+					add_openvpn_arg (args, "tcp6-client");
+				else if (NM_IN_STRSET (proto, NMOVPN_PROTCOL_TYPES))
+					add_openvpn_arg (args, proto);
 				else {
 					g_set_error (error,
 					             NM_VPN_PLUGIN_ERROR,
@@ -1740,6 +1747,15 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 		add_openvpn_arg (args, tmp);
 	}
 
+	/* ifconfig */
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP);
+	tmp2 = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP);
+	if (tmp && tmp2) {
+		add_openvpn_arg (args, "--ifconfig");
+		add_openvpn_arg (args, tmp);
+		add_openvpn_arg (args, tmp2);
+	}
+
 	/* Punch script security in the face; this option was added to OpenVPN 2.1-rc9
 	 * and defaults to disallowing any scripts, a behavior change from previous
 	 * versions.
@@ -1799,30 +1815,6 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 			if (tmp && tmp[0])
 				add_openvpn_arg (args, tmp);
 		}
-
-		add_openvpn_arg (args, "--ifconfig");
-
-		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_LOCAL_IP);
-		if (!tmp) {
-			/* Insufficient data (FIXME: this should really be detected when validating the properties */
-			g_set_error_literal (error,
-			                     NM_VPN_PLUGIN_ERROR,
-			                     NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			                     _("Missing required local IP address for static key mode."));
-			return FALSE;
-		}
-		add_openvpn_arg (args, tmp);
-
-		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_IP);
-		if (!tmp) {
-			/* Insufficient data (FIXME: this should really be detected when validating the properties */
-			g_set_error_literal (error,
-			                     NM_VPN_PLUGIN_ERROR,
-			                     NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
-			                     _("Missing required remote IP address for static key mode."));
-			return FALSE;
-		}
-		add_openvpn_arg (args, tmp);
 	} else if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD)) {
 		/* Client mode */
 		add_openvpn_arg (args, "--client");
@@ -1949,7 +1941,7 @@ check_need_secrets (NMSettingVpn *s_vpn, gboolean *need_secrets)
 	if (!strcmp (ctype, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
 		/* Will require a password and maybe private key password */
 		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
-		key = nmv_utils_str_utf8safe_unescape_c (key, &key_free);
+		key = nm_utils_str_utf8safe_unescape (key, &key_free);
 		if (is_encrypted (key) && !nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS))
 			*need_secrets = TRUE;
 
@@ -1972,7 +1964,7 @@ check_need_secrets (NMSettingVpn *s_vpn, gboolean *need_secrets)
 	} else if (!strcmp (ctype, NM_OPENVPN_CONTYPE_TLS)) {
 		/* May require private key password */
 		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
-		key = nmv_utils_str_utf8safe_unescape_c (key, &key_free);
+		key = nm_utils_str_utf8safe_unescape (key, &key_free);
 		if (is_encrypted (key) && !nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS))
 			*need_secrets = TRUE;
 	} else {
